#!/usr/bin/ruby1.8 -w

###
# swconf.rb
# Ruby telnet version of the XorPlus range configuration script. 
###

require 'lb9a.rb'
require 'logger'
require 'optparse'


$options = Hash.new()
$optparse = OptionParser.new do |opts|
        #Banner
        opts.banner = "Allows configuration of multiple ports on a Quanta running XorPlus.\nUsage: ./swconf.rb [host] [port]"
                $options[:dbg] = false
                opts.on('-d','--debug','Enable verbose debugging mode (default: off)') do
                        $options[:dbg] = true
                end
		$options[:dr] = false
		opts.on('-r','--dryrun','Do a dry run (Do not actually send command - default: off)') do
                        $options[:dr] = true
                end
                opts.on( '-h', '--help', 'Display this list' ) do
                        puts opts
                        exit
                end
        end
$host,$port = *$optparse.parse!

def main_help()
        puts "\tclear - clear the screen"
        puts "\tdelete - delete settings"
        puts "\texit - exit script"
        puts "\thelp - display this list"
        puts "\tiface [eth0|eth1] [IP addr] - Configure management interface address. Address is in CIDR form (e.g 172.16.0.30/16)"
        puts "\tports [portrange] - Configure VLAN and mode of ports in a list of ports [portrange]"
        puts "\tsave - Commit changes to memory"
        puts "\tvlan [VLAN ID] - instantiate a VLAN interface of tag [VLAN ID]"
end

def ports_help()
        puts "\tvlan [1..1020]: set native vlan for ports"
        puts "\tmode [access|trunk]"
        puts "\tdelete: wipe out port settings"
        puts "\trange [portrange]: configure new range of ports"
        puts "\texit: exit from port context"
end

def prompt(prompt)
        print prompt
        ret = gets.chomp
        return ret
end

if __FILE__ == $0
	LOG.debug "host: #{$host} port: #{$port}"
	@saved = false	
	@changed = false	# to true if any configurations are made	

	#initialize session
	@sn = Session.new($host, $port, $options[:dbg], $options[:dr])
	@sn.init 
	LOG.debug "Session: #{@sn}"

	#clear ARGV for gets
        ARGV.clear	

	#get user input, primary interface
	resp = prompt("> ").split(' ')	
	rlen = resp.length

	#CLI loop
	while (resp[0] != "exit")
		case resp[0]
			when "clear"	#clear terminal
				system("clear")
			when "iface"
				if rlen == 3 
					iface = resp[1]
					addr = resp[2]
					@changed = @sn.set_iface(iface, addr)
				else
					puts "\tiface [eth0|eth1] [IP addr/mask]"
				end
			when "vlan"
				if rlen == 2
					id = resp[1].to_i
					@changed = @sn.set_vlan(id)
				else
					puts "\tvlan [VLAN ID]"
				end
			when "ports"
				if rlen == 2
					parr = resp[1]

                                       	ilist = parr.to_ra.to_rng.sort.uniq 
					ctxt = prompt("(#{resp[0]})# ")
					
					while ctxt != "exit"
						# range [portslist] context 
						if ctxt.include? "range"
							ilist = ctxt.split(' ')[1].to_ra.to_rng.sort.uniq 
							LOG.debug("swconf.rb:ilist:#{ilist}")
						elsif ctxt.include? "help"
							ports_help
						elsif ctxt.include? "clear"
							system("clear")
						else
							@changed = @sn.set_ports(ctxt, ilist)
						end
						ctxt = prompt("(#{resp[0]})# ")
					end
				else
					puts "ports [portrange]"
					puts "[portrange] is a string of ports you want to configure (e.g. \"1..4,7,23\")"
				end
			when "delete"
				if rlen >= 3
					if resp[1] == "vlan"
						@changed = @sn.set_vlan(resp[2].to_i, true)
					elsif resp[1] == "iface"
						@changed = @sn.set_iface(resp[2], resp[3], true)
					else
						puts "swconf.rb:#{resp[1]}:unknown parameter"
					end
				else
					puts "\tdelete iface [eth0|eth1] [address/mask] - delete management port settings"
					puts "\tdelete vlan [VLAN ID] - delete VLANS"
				end
			when "save"
				@sn.commit
				@saved = true
			when "help" 
				main_help
			else
				puts "\tUnknown command"
		end

		resp.clear
		resp = prompt("> ").split(' ')	
		rlen = resp.length
	end
	
	if (@changed && !@saved)   
		print "Exit without commit? ([y]/n): " 
		if (gets.chomp == 'n') then @sn.commit end
	end

	puts "Logging off"
	@sn.logoff(@saved)
end




